local Tags={}
Relog.Tags=Tags

local TagCompiler = AceLibrary("TagCompiler-1.0")

local PowerColors={}
local ClassColors={}

for i=0,4 do
	PowerColors[i]=string.format("|cff%02x%02x%02x",255*ManaBarColor[i].r,255*ManaBarColor[i].g,255*ManaBarColor[i].b)
end

for k,v in pairs(RAID_CLASS_COLORS) do
	ClassColors[k]=string.format("|cff%02x%02x%02x",255*v.r,255*v.g,255*v.b)
end

local TagData = {
	Time = { tableData = "Time" },
	Type = { tableData = "Type" },
	Source = { tableData = "Source" },
	Victim = { tableData = "Victim" },
	Ability = { tableData = "Ability" },
	VictimAbility = { tableData = "VictimAbility" },
	Result = { tableData = "Result" },

	Amount = { method = function(tP, prior) if tP.Type=="Item" and tP.ItemLink==false then
							local G,S,C
							G=math.floor(tP.Amount/COPPER_PER_GOLD)
							S=math.fmod(math.floor(tP.Amount/COPPER_PER_SILVER),SILVER_PER_GOLD)
							C=math.fmod(tP.Amount,COPPER_PER_SILVER)
							if G>0 then
								return string.format("%d|cffe8cf4fg|r %d|cff9fa49fs|r %d|cffa87048c|r",G,S,C)
							elseif S>0 then
								return string.format("%d|cff9fa49fs|r %d|cffa87048c|r",S,C)
							else
								return string.format("%d|cffa87048c|r",C)
							end
						elseif tP.Type=="Item" then
							return tP.Amount.."x"
						else
							return tP.Amount
						end
					end },
	RawAmount = { tableData = "Amount" },

	Attribute ={ tableData = "Attribute"},



	--Leech Info
	SourceGained={ tableData = "SourceGained"},
	AttributeGained={ tableData = "AttributeGained"},
	AmountGained={ tableData = "AmountGained"},

	--Reputation
	RepFaction={ tableData = "Faction"},
	Rank={ tableData = "Rank"},

	--Special Data for Damage
	DamageType={ tableData = "DamageType"},
	Absorb={ tableData = "AbsorbAmount"},
	Block={ tableData = "BlockAmount"},
	Resist={ tableData = "ResistAmount"},
	Vulnerable={ tableData = "VulnerableAmount"},

	--Healing
	Overheal={ tableData = "Overheal"},

	--Item Data
	ItemLink={ tableData = "ItemLink"},
	ItemName={ tableData = "ItemName"},
	ItemRarity={ tableData = "ItemRarity"},
	ItemString={ tableData = "ItemString"},

	--Items just for Experience which requires several
	BonusAmount={  tableData = "BonusAmount"},
	BonusType={ tableData = "BonusType"},
	PenaltyAmount={ tableData = "PenaltyAmount"},
	PenaltyType={ tableData = "PenaltyType"},
	AmountRaidPenalty={ tableData = "AmountRaidPenalty"},
	AmountGroupBonus={ tableData = "AmountGroupBonus"},


	--Unit Functions
	UnitID = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.UnitID end },
	Friendly = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Friendly end },
	isPlayer = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.isPlayer end },
	isGrouped = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.isGrouped end },
	Class = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.lClass end },
	Faction = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Faction end },
	Level = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Level end },

	Health = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Health end },
	HealthMax = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.HealthMax end },
	Power = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.Power end },
	PowerMax = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.PowerMax end },
	PowerType = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.PowerType end },
	MinRange = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.MinRange end },
	MaxRange = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.MaxRange end },

	RankName = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.RankName end },
	RankNumber = { method = function(tP, prior) local U=Tags.GetUnit(prior); return U.RankNumber end },

	FriendlyC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if type(prior)~="string" then return "" end if U.Friendly then return "|cff40ff40"..prior.."|r";else return "|cffff4040"..prior.."|r" end end },
	ClassC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Class then return ClassColors[U.Class]..prior.."|r";else return prior end end },

	--Start Color Commands
	StartFriendlyC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Friendly then return "|cff40ff40";else return "|cffff4040" end end },
	StartClassC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.Class then return ClassColors[U.Class];else return "" end end },
	StartPowerC = { method = function(tP, prior) local U=Tags.GetUnit(prior); if U.PowerType then return PowerColors[U.PowerType];else return "" end end },

	--Date Command Same as normal except %s=Milliseconds
	Date = { method = function(tP, prior,arg1,arg2) local SubSecond=string.format("%03d",1000*(arg2-math.floor(arg2)));arg1=string.gsub(arg1,"%%s",SubSecond);return date(arg1,math.floor(arg2)) end,
		parameterTypes = {"string", "number"},	},

	Player = { method = function(tP, prior) return UnitName("player") end },
	["!TagError"] = function(x) Relog:Print(x) end, 
}

--\"%H:%M:%S.%s\"
--[Date(\"%H:%M:%S.%s\",Time)] 
local DefaultTags={
	Buff =		"[Victim:ClassC] is buffed with [Ability]([Amount])",	
	Cast =		"[Source:ClassC] [Result:\"begins to \"][Amount:\"performs \"][Ability][Victim:(\" on \"..Victim:ClassC)]",
	Damage =	"[Source:ClassC]'s [Ability] [Result] [Victim:ClassC..\"(\"..Victim:Health..\"/\"..Victim:HealthMax..\")\"] for [Amount] [DamageType~=\"Physical\":DamageType][(Block>0):(\" (Block: \"..Block..\")\")][(Absorb>0):(\" (Absorb: \"..Absorb..\")\")][(Resist>0):(\" (Resisted: \"..Resist..\")\")][(Vulnerable>0):(\" (Vulnerable: \"..Vulnerable..\")\")]",
	Death =		"[Victim:ClassC] killed [Source:\"by \"][Source:ClassC][Ability:\"'s \"][Ability:Ability]",
	Debuff =	"[Victim:ClassC] is debuffed with [Ability]([Amount])",
	Dispel =	"[Source:ClassC]'s [Ability] removes [Victim:ClassC]'s [VictimAbility]",
	Durability =	"[Source:ClassC]'s [Ability] damages [Victim:ClassC]'s [ItemLink]",
	Drain =		"[Source:ClassC]'s [Ability] drains [Victim:ClassC]'s [Amount] [Attribute]",
	Enchant =	"[Source:ClassC] enchants [Victim:ClassC]'s [ItemLink] with [Ability]",
	Environment =	"[Victim:ClassC] [Result] for [Amount] [DamageType]",
	Experience =	"Exp: [Amount] from [Source:ClassC]([Source:Level])",	
	["Extra Attack"] = "[Victim:ClassC] gains [Amount] extra attack[(Amount>1):\"s\"] from [Ability]",
	Fade =		"[Victim:ClassC]'s [Ability] fades",
	Fail =		"[Source:ClassC]'s [Ability] fails due to [Result]",
	["Feed Pet"] =	"[Victim:ClassC] feeds their pet [ItemLink]",
	Gain =		"[Victim:ClassC] gains [Amount] [Attribute] from [Source:ClassC]'s [Ability]",
	Heal =		"[Source:ClassC]'s [Ability] [Result] [Victim:ClassC] for [Amount] [(Overheal>0):(\"(Overheal: \"..Overheal..\")\")]",	
	Honor =		"Honor: [Amount] from [Source:ClassC] [Source:RankName]",	
	Interrupt =	"[Source:ClassC] interrupts [Victim:ClassC]'s [Ability]",
	Item =		"[Source:ClassC] [Result]s [Amount] [ItemLink]",
	Leech =		"[Source:ClassC]'s [Ability] leeches [Amount] [Attribute] from [Victim:ClassC] to [SourceGained:ClassC] as [AmountGained] [AttributeGained]",
	Miss =		"[Source:ClassC][Ability:\"'s \"][Ability:Ability] [Result] [Victim:ClassC]",
	Reputation =	"Rep: [Ability:Ability] [Amount] with [Faction]([Rank])",
}

local CurTime
function Tags.CompileLog(num)
	local log=Relog:GetLogRaw(num)

	CurTime=log.Time
	Tags.ResetCache()

	if DefaultTags[log.Type] then
		Relog.db.char.Log.CompiledCache[num]=TagCompiler:Evaluate(DefaultTags[log.Type], TagData, "string", log)
		if log.Type=="Death" or TagCompiler:Evaluate("[(Victim==Player)|(Source==Player):\"true\"]", TagData, "string", log)~="" then
			if Relog.db.profile.OutputLog then
				getglobal("ChatFrame"..Relog.db.profile.OutputFrame):AddMessage(Relog.db.char.Log.CompiledCache[num])
			end
		end
	else
		
		if log.Type then
			Relog:CustomPrint(1,1,1,ChatFrame3,nil,nil,"No String for "..log.Type)
		end
	end
end

function Tags.CompileTag(tag,table)
	CurTime=table.Time
	Tags.ResetCache()
	return TagCompiler:Evaluate(tag, TagData, "string", table)
end

local UnitCache={}
local NumInCache=0

function Tags.ResetCache()
	for k in pairs(UnitCache) do
		UnitCache[k]=nil
	end

	NumInCache=0
end

local Previous

function Tags.GetUnit(name)
	if name==true then
		name=Previous
	else
		Previous=name
	end

	local Unit=UnitCache[name]

	if Unit==nil then
		NumInCache=NumInCache+1
		UnitCache[name]=Relog.Units:GetUnit(name,CurTime,NumInCache)
		Unit=UnitCache[name]
	end

	return Unit	
end